@@ -3,15 +3,13 @@ Created on 2022-09-17
 
 @author: wf
 '''
-from difflib import Differ
-import datetime
 import importlib
 import os
 import re
 import typing
 from jpcore.justpy_app import JustpyApp
 from starlette.routing import request_response
-from jpcore.example import ExampleSource
+from jpcore.example import Example,ExampleSource
 
 class JustpyDemoApp:
     """
@@ -40,6 +38,8 @@ class JustpyDemoApp:
         self.source_link=f"""<a href="{url}" target="_blank">{self.source_file}</a>"""
         self.video_url=None
         self.try_it_url=None
+        self.issue = None
+        self.fixed = None
         self.name=os.path.basename(self.source_file).replace(".py","")
         self.wpfunc = wpfunc
         self.debug=debug
@@ -98,35 +98,17 @@ class JustpyDemoApp:
         video_link=f"""<a href="{target_url}" target="_blank"><img src="{video_url}" alt="{alt}" style="width:{video_size}px;height:{video_size}px;"></a>"""
         return video_link
     
-    def update_from_tutorial(self,tutorial_source:ExampleSource,target_path:str=None):
+    def update_from_tutorial_example(self,tutorial_example:Example,target_path:str=None):
         """
-        update me from the given tutorial_source
+        update me from the given tutorial
         
         Args:
-            tutorial_source(ExampleSource): - the tutorial content to update me from 
+            tutorial(Tutorial): - the tutorial content to update me from 
             target_path(str): where to store the updated result - if None store at the original location
         """
         if target_path is None:
             target_path=self.pymodule_file
-        now = datetime.datetime.now(tz=datetime.timezone.utc)
-        timestamp=now.isoformat()
-        comment=f"""# Justpy Tutorial demo {self.name} 
-# generated by demoapp.update_from tutorial at {timestamp} 
-# see {self.example_source.url}
-# 
-# {tutorial_source.description}            
-# see {tutorial_source.url}"""                
-        with open(target_path, 'w') as code:
-            print(comment, file=code)
-            for line in tutorial_source.lines:
-                if line.startswith("jp.justpy("):
-                    args=line.replace("jp.justpy(","")
-                    demo_start=f"""# initialize the demo
-from examples.basedemo import Demo
-Demo("{self.name}", {args}"""
-                    print(demo_start,file=code)
-                else:
-                    print(line,file=code)
+        tutorial_example.write_as_demo(target_path)
                             
     def same_as_tutorial(self,tutorial_source:ExampleSource,debug:bool=False):
         """
@@ -136,42 +118,7 @@ Demo("{self.name}", {args}"""
             tutorial_source(ExampleSource): - the tutorial content to update me from 
             debug(bool): if True show debug information
         """
-        d=Differ()
-        demo_source=self.example_source
-        if debug:
-            print(self)
-            print(tutorial_source)
-            print(demo_source)
-        diff=d.compare(tutorial_source.lines,demo_source.lines)
-        problems=[]
-        for i,diff_line in enumerate(diff):
-            if i==0:
-                if not "# Justpy Tutorial demo" in diff_line:
-                    problems.append("Justpy Tutorial demo comment in line 1 missing")
-            elif diff_line.startswith("-"):
-                if diff_line.startswith("- # generated by demoapp"):
-                    pass
-                elif diff_line.startswith("- jp.justpy("):
-                    pass
-                else:
-                    problems.append(f"removed line {i+1}:{diff_line}")
-            elif diff_line.startswith("+"):
-                if diff_line.startswith("+ #"):
-                    pass
-                elif diff_line.startswith("+ # generated by demoapp"):
-                    pass
-                elif diff_line.startswith("+ from examples.basedemo import Demo"):
-                    pass
-                elif diff_line.startswith("+ Demo"):
-                    pass
-                elif diff_line.startswith("+ from  examples.basedemo import Demo"):
-                    pass
-                elif diff_line[2:].strip():
-                    problems.append(f"added line {i+1}:{diff_line}")
-            if debug:
-                print(f"  {i+1:3}:{diff_line}")
-        return problems
-        
+        return self.example_source.same_as(tutorial_source,debug=debug)
 
     def mount(self, app:JustpyApp, name:str=None):
         """
@@ -184,7 +131,7 @@ Demo("{self.name}", {args}"""
         demo_module = importlib.import_module(self.pymodule)
         self.wpfunc = getattr(demo_module, self.wpfunc_name)
         if self.wpfunc is None:
-            raise Exception(f"can't start {self.pymodule_file} -wpfunc/endoint is None")
+            raise Exception(f"can't start {self.pymodule_file} -wpfunc/endpoint is None")
         # https://fastapi.tiangolo.com/advanced/sub-applications/
         endpoint=app.response(self.wpfunc)
         # wrap/cast the endpoint for starlettes routing
