@@ -11,6 +11,8 @@ import logging
 import os
 import pathlib
 import socket
+from enum import Enum
+
 import psutil
 import sys
 import traceback
@@ -20,6 +22,7 @@ from sys import platform
 from multiprocessing import Process
 from threading import Thread
 from starlette.applications import Starlette
+from starlette.requests import Request
 from starlette.routing import Route
 from starlette.endpoints import HTTPEndpoint
 from starlette.responses import HTMLResponse, JSONResponse,PlainTextResponse, Response
@@ -32,6 +35,12 @@ from jpcore.template import Context
 from jpcore.webpage import WebPage
 from itsdangerous import Signer
 
+# typing shorthand notations
+WebPage_or_Response = typing.Union[typing.Union[WebPage, Response], typing.Awaitable[typing.Union[WebPage, Response]]]
+Jp_Route_Callback = typing.Union[
+    typing.Callable[[typing.Optional[Request]], WebPage_or_Response],
+    typing.Callable[[], WebPage_or_Response]
+]
 # TODO refactor to object oriented version where this is a property of some instance of some class
 cookie_signer = Signer(str(jpconfig.SECRET_KEY))
 
@@ -82,31 +91,42 @@ template_options = {
     "vega": jpconfig.VEGA,
 }
 
-async def handle_event(data_dict, com_type=0, page_event=False):
+
+class CommunicationType(Enum):
+    """
+    communication types supported by justpy
+    """
+    WEBSOCKET = 0
+    AJAX = 1
+
+
+async def handle_event(
+        data_dict: dict,
+        com_type: CommunicationType = CommunicationType.WEBSOCKET,
+        page_event: bool = False
+) -> typing.Optional[dict]:
     """
     handle the given event
     
     Args:
         data_dict(dict): the dict with the data
-        com_type(int):  the communication type - default: 0
+        com_type(CommunicationType):  the communication type - default: WEBSOCKET
         page_event(bool): if True handle as a page event
     """
-    # com_type 0: websocket, con_type 1: ajax
-    connection_type = {0: "websocket", 1: "ajax"}
     logging.info(
-        "%s %s %s", "In event handler:", connection_type[com_type], str(data_dict)
+        "%s %s %s", "In event handler:", com_type.WEBSOCKET.name, str(data_dict)
     )
     event_data = data_dict["event_data"]
+    page_id = event_data["page_id"]
     try:
-        p = WebPage.instances[event_data["page_id"]]
+        p = WebPage.instances[page_id]
     except:
         logging.warning("No page to load")
         return
     event_data["page"] = p
-    if com_type == 0:
-        event_data["websocket"] = WebPage.sockets[event_data["page_id"]][
-            event_data["websocket_id"]
-        ]
+    if com_type is CommunicationType.WEBSOCKET:
+        websocket_id = event_data["websocket_id"]
+        event_data["websocket"] = WebPage.sockets[page_id][websocket_id]
     # The page_update event is generated by the reload_interval Ajax call
     if event_data["event_type"] == "page_update":
         build_list = p.build_list()
@@ -157,18 +177,18 @@ async def handle_event(data_dict, com_type=0, page_event=False):
 
     # If page is not to be updated, the event_function should return anything but None
     if event_result is None:
-        if com_type == 0:  # WebSockets communication
+        if com_type is CommunicationType.WEBSOCKET:  # WebSockets communication
             if jpconfig.LATENCY:
                 await asyncio.sleep(jpconfig.LATENCY / 1000)
             await p.update()
-        elif com_type == 1:  # Ajax communication
+        elif com_type is CommunicationType.AJAX:  # Ajax communication
             build_list = p.build_list()
     try:
         if c is not None:
             after_result = await c.run_event_function("after", event_data, True)
     except:
         pass
-    if com_type == 1 and event_result is None:
+    if com_type is CommunicationType.AJAX and event_result is None:
         dict_to_send = {
             "type": "page_update",
             "data": build_list,
@@ -214,7 +234,13 @@ class JustpyApp(Starlette):
             text+=f"func: {route.endpoint.__name__}"
         return text
     
-    def add_jproute(self,path:str,wpfunc:typing.Callable,name:str=None):
+    def add_jproute(
+            self,
+            path: str,
+            wpfunc: Jp_Route_Callback,
+            name: typing.Optional[str] = None,
+            methods: typing.Optional[typing.List[str]] = None
+    ) -> typing.Callable:
         """
         add a route for the given Webpage returning func
         
@@ -222,48 +248,49 @@ class JustpyApp(Starlette):
             path(str): the path to use as route
             wpfunc(typing.Callable): a Webpage returning func
             name(str): the name of the route
+            methods:
         """
         endpoint=self.response(wpfunc)
         if name is None:
             name=wpfunc.__name__
-        self.router.add_route(path,endpoint,name=name,include_in_schema=False)
+        self.router.add_route(path, endpoint, methods=methods, name=name, include_in_schema=False)
+        return endpoint
     
-    def jproute(self,
-        path: str,
-        name: typing.Optional[str] = None)-> typing.Callable:  # pragma: nocover
+    def jproute(
+            self,
+            path: str,
+            name: typing.Optional[str] = None,
+            methods: typing.Optional[typing.List[str]] = None
+    ) -> typing.Callable:  # pragma: nocover
         """ 
         justpy route decorator
         
         function will we "wrapped" as a response and a route added
         
         Args:
-            func(typing.Callable): the function to convert to a reponse
+            path: the path to use as route
+            name: the name of the route
+            methods:
         """
         
-        def routeResponse(func:typing.Callable)-> typing.Callable:
+        def routeResponse(func: typing.Callable) -> typing.Callable:
             """
             decorator for the given func
             
             Args:
-                func(typing.Callable)
+                func(typing.Callable): the function to convert to a reponse
                 
             Returns:
                 Callable: an endpoint that has been routed
             
             """
-            endpoint=self.response(func)
-            self.router.add_route(
-                path,
-                endpoint,
-                name=name if name is not None else func.__name__,
-                include_in_schema=False,
-            )
+            endpoint = self.add_jproute(path=path, name=name, wpfunc=func, methods=methods)
             self.route(path)
             return endpoint
         
         return routeResponse
     
-    def response(self,func:typing.Callable):
+    def response(self, func: Jp_Route_Callback) -> typing.Callable:
         """
         response decorator converts a function to a response
         
@@ -272,8 +299,11 @@ class JustpyApp(Starlette):
         
         Args:
             func(typing.Callable): the function (returning a WebPage) to convert to a response
+
+        Returns:
+
         """
-        async def funcResponse(request)->HTMLResponse:
+        async def funcResponse(request: Request) -> Response:
             """
             decorator function to apply the function to the request and
             return it as a response
@@ -282,13 +312,17 @@ class JustpyApp(Starlette):
                 request(Request): the request to apply the function to
                 
             Returns:
-                Response: a HTMLResponse applying the justpy infrastructure
+                Response: a Response applying the justpy infrastructure
             
             """
             new_cookie = self.handle_session_cookie(request)
-            wp = await self.get_page_for_func(request, func)
-            response = self.get_response_for_load_page(request, wp)
-            response = self.set_cookie(request, response, wp, new_cookie)
+            wp_or_response = await self.get_page_for_func(request, func)
+            if isinstance(wp_or_response, WebPage):
+                wp = wp_or_response
+                response = self.get_response_for_load_page(request, wp)
+                response = self.set_cookie(request, response, wp, new_cookie)
+            else:
+                response = wp_or_response
             if jpconfig.LATENCY:
                 await asyncio.sleep(jpconfig.LATENCY / 1000)
             return response
@@ -297,7 +331,11 @@ class JustpyApp(Starlette):
         # parameter in the funcResponse later when applied 
         return funcResponse
 
-    async def get_page_for_func(self, request, func:typing.Callable)->WebPage:
+    async def get_page_for_func(
+            self,
+            request: Request,
+            func: Jp_Route_Callback
+    ) -> typing.Union[WebPage, Response]:
         """
         get the Webpage for the given func
 
@@ -325,7 +363,7 @@ class JustpyApp(Starlette):
                 load_page = func_to_run()
         return load_page
 
-    def get_response_for_load_page(self,request,load_page):
+    def get_response_for_load_page(self, request: Request, load_page: WebPage) -> Response:
         """
         get the response for the given webpage
         
@@ -335,7 +373,7 @@ class JustpyApp(Starlette):
             return as a full HtmlResponse
         
         Returns:
-            Reponse: the response for the given load_page
+            Response: the response for the given load_page
         """
         page_type = type(load_page)
         assert issubclass(
@@ -382,7 +420,7 @@ class JustpyApp(Starlette):
         response = templates.TemplateResponse(load_page.template_file, context)
         return response
     
-    def handle_session_cookie(self,request) -> typing.Union[bool, Response]:
+    def handle_session_cookie(self, request) -> typing.Union[bool, Response]:
         """
         handle the session cookie for this request
         
@@ -409,7 +447,13 @@ class JustpyApp(Starlette):
                 logging.debug(f"New session_id created: {request.session_id}")
         return new_cookie
     
-    def set_cookie(self, request, response, load_page, new_cookie: typing.Union[bool, Response]):
+    def set_cookie(
+            self,
+            request: Request,
+            response: Response,
+            load_page: WebPage,
+            new_cookie: typing.Union[bool, Response]
+    ) -> Response:
         """
         set the cookie_value
         
@@ -431,6 +475,7 @@ class JustpyApp(Starlette):
                 response.set_cookie(k, v, max_age=jpconfig.COOKIE_MAX_AGE, httponly=True)
         return response
 
+
 class JustpyAjaxEndpoint(HTTPEndpoint):
     """
     Justpy specific HTTPEndpoint/app (ASGI application)
@@ -463,7 +508,7 @@ class JustpyAjaxEndpoint(HTTPEndpoint):
         msg_type = data_dict["type"]
         data_dict["event_data"]["msg_type"] = msg_type
         page_event = True if msg_type == "page_event" else False
-        result = await handle_event(data_dict, com_type=1, page_event=page_event)
+        result = await handle_event(data_dict, com_type=CommunicationType.AJAX, page_event=page_event)
         if result:
             if jpconfig.LATENCY:
                 await asyncio.sleep(jpconfig.LATENCY / 1000)
@@ -478,6 +523,7 @@ class JustpyAjaxEndpoint(HTTPEndpoint):
         ].on_disconnect()  # Run the specific page disconnect function
         return JSONResponse(False)
 
+
 class JustpyServer:
     """
     a justpy Server
@@ -524,7 +570,7 @@ class JustpyServer:
         self.running = False
         
     @classmethod
-    def getDefaultHost(cls):
+    def getDefaultHost(cls) -> str:
         """
         get the default host as the fully qualifying hostname
         of the computer the server runs on
@@ -539,14 +585,14 @@ class JustpyServer:
             # host="127.0.0.1"
         return host
 
-    async def start(self, wpfunc,websockets: bool = True, **kwargs):
+    async def start(self, wpfunc, websockets: bool = True, **kwargs):
         """
         start a justpy server for the given webpage function wpfunc
 
         Args:
             wpfunc: the (async) function for the webpage
-
-
+            websockets: If True use websockets for server/client communication
+            kwargs:
         """
         # this import is actually calling code ...
         import justpy as jp
@@ -605,7 +651,7 @@ class JustpyServer:
                 child.terminate()
             self.proc.terminate()
 
-    def next_server(self):
+    def next_server(self) -> 'JustpyServer':
         """
         get another similar server with the port number incremented by one
         """
@@ -618,7 +664,7 @@ class JustpyServer:
         )
         return next_server
 
-    def get_url(self, path):
+    def get_url(self, path: str) -> str:
         """
         get the url for the given path
 
